package cz.upol.jj2.lab09.cars;

import java.time.LocalDate;
import java.util.*;

/**
 * jednoduchá "databáze" - pár in-memory seznamů
 * předvyplněná verze s daty
 * pár "factory" metod
 * pár CRUD metod
 * není zde nic nového, jen ať máme nějaká data a přitom neřešíme napojovani dalších pluginů a chystání DB
 */
public class SimpleDatabase {
    private List<Car> cars;
    private List<Person> persons;

    private List<HistoryItem> histories;

    public static SimpleDatabase emptyDBFactory(){
        return new SimpleDatabase(new ArrayList<>(), new ArrayList<>(), new ArrayList<>());
    }

    public static SimpleDatabase prefilledDBFactory(){
        List<Person> persons = new ArrayList<>();
        List<Car> cars = new ArrayList<>();
        List<HistoryItem> histories = new ArrayList<>();
        Person[] perArray = {
                new Person(1, "James", "Kirk", LocalDate.of(2233,3,22)),
                new Person(2, "Wesley", "Crusher",LocalDate.of(2348,1,1)),
                new Person(3, "Nyota", "Uhura", LocalDate.of(2233,1,19))
        };
        persons.addAll(Arrays.stream(perArray).toList());

        HistoryItem[] histArray = {
                new HistoryItem(4,2248, 2255, perArray[0]),
                new HistoryItem(5,2256, 2263, perArray[2]),
                new HistoryItem(6,2263, 2270, perArray[1]),
                new HistoryItem(7,2248, 2300, perArray[0])
        };
        histories.addAll(Arrays.stream(histArray).toList());

        Car[] carArray = {
                new Car(8, 1995, "Škoda", "Favorit", 300000, true, new ArrayList<>()),
                new Car(9, 2000,"Škoda", "Felicia", 1, false, new ArrayList<>()),
                new Car(10, 2020,"Škoda", "Superb", 0, false, new ArrayList<>())
        };

        carArray[0].addToHistory(histArray[0]);
        carArray[0].addToHistory(histArray[1]);
        carArray[0].addToHistory(histArray[2]);

        carArray[1].addToHistory(histArray[3]);

        cars.addAll(Arrays.stream(carArray).toList());
        return new SimpleDatabase(cars, persons, histories);
    }

    private SimpleDatabase(List<Car> cars, List<Person> persons, List<HistoryItem> histories) {
        this.cars = cars;
        this.persons = persons;
        this.histories = histories;
    }

    public List<Car> getCars() {
        return List.copyOf(cars);
    }

    public List<Person> getPersons() {
        return List.copyOf(persons);
    }

    public List<HistoryItem> getHistories() {
        return List.copyOf(histories);
    }

    public Optional<Car> carById(int id){
        return cars.stream().filter(car -> car.getId() == id).findFirst();
    }

    public Optional<Person> personById(int id){
        return persons.stream().filter(p -> p.getId() == id).findFirst();
    }

    public boolean addPerson(Person p){
        var isInDb = personById(p.getId());
        if (isInDb.isEmpty()){
            return persons.add(p);
        }
        return false;
    }

    public boolean addCar(Car c){
        var isInDb = carById(c.getId());
        if (isInDb.isEmpty()){
            return cars.add(c);
        }
        return false;
    }

    public boolean dropPerson(int id){
        var p = personById(id);
        if (p.isEmpty()){ //neexsitující osoba
            return false;
        }
        Person person = p.get();
        if (histories.stream().anyMatch( h-> h.owner().equals(person))) { // na osobu je navázána historie auta
            return false;
        }
        return persons.remove(p.get());
    }

    public boolean dropCar(int id){
        var c = carById(id);
        if (c.isEmpty()){
            return false;
        }
        Car car = c.get();
        for (HistoryItem h : car.getHistory()) {
            histories.remove(h);
        }
        return cars.remove(car);
    }

    public void print() {
        StringBuilder sb = new StringBuilder();
        sb.append("DATABÁZE");
        sb.append(System.lineSeparator());
        sb.append("Osoby:");
        sb.append(System.lineSeparator());
        for (var p : persons) {
            sb.append(p);
            sb.append(System.lineSeparator());
        }
        sb.append("Vlastnické záznamy:");
        sb.append(System.lineSeparator());
        for (var h : histories) {
            sb.append(h);
            sb.append(System.lineSeparator());
        }
        sb.append(System.lineSeparator());

        sb.append(System.lineSeparator());
        sb.append("Auta:");
        sb.append(System.lineSeparator());
        for (var c : cars) {
            sb.append(c);
            sb.append(System.lineSeparator());
        }
        sb.append(System.lineSeparator());
        System.out.println(sb.toString());
    }
}
